@@ -95,6 +95,7 @@ class CertEntry(object):
         self.sig = b""
 
         self.chain = []
+        self.vcerts = []                # generated by verify, computed certchain, includes trusted.
 
         self.vis_map = {}
         self.default_vismap = dict(schema=CERT_SCHEMA,
@@ -106,14 +107,16 @@ class CertEntry(object):
 
     # ============== Metadata access functions =============================================
 
-    def chain_certs(self):
-        return reversed([i["cert"] for i in self.chain if "cert" in i])
+    def vchain(self):
+        if len(self.vcerts) > 1 and self.vcerts[0] == self.vcerts[1]:
+            return self.vcerts[1:]      # verify() can double-add the root cert under certain
+        return self.vcerts              # circumstances (link name vs append etc)
 
-    def chain_types(self):          # NOTE This doesn't include linked-by-name and it needs to.D
-        return [i.get("cert_type", "") for i in self.chain_certs()]
+    def vtypes(self):
+        return "/"+"/".join([i.get("cert_type", "") for i in self.vchain()])
 
-    def chain_names(self):
-        return [i["subject_name"] for i in self.chain_certs()]
+    def vnames(self):
+        return "/"+"/".join([i["subject_name"] for i in self.vchain()])
 
 
     # ============== Private key encrypt ===================================================
